
/*
  TexDemo2 is a Win32 consol application that
  displays a textured rectangle and zooms it in and out
  demonstrating trilinear filtering, this second version of
  textdemo have some small modifications so that it uses gouraud
  shading at the same time. Note that this demo
  requires voodoo graphics with 2 texture units.
  Original voodoo graphics only have 1 texture unit but
  the new voodoo2 has 2! Because this demo requires voodoo2
  it is run in 800*600 since it can't be run on original
  voodoo graphics chipset anyway.
*/

#include <glide.h>

void main(void)
{
  GrHwConfiguration HwConfig;
  Gu3dfInfo TexInfo;
  GrTexInfo MipMap1;
  FxU32 TexStartAddress1;
  FxU32 TexStartAddress2;
  float w;
  GrVertex Rectangle[4];
  int vlist[4];
  float size, increment;

  // Initialise Glide - this function MUST be called
  // before anything else is done with Glide:
  grGlideInit();

  // Check for precence of 3dfx hardware:
  if ((grSstQueryHardware(&HwConfig)) == FXFALSE)
  {
        printf("Error! No 3dfx hardware!");
		exit(1);
  }
  // found 3dfx hardware!
  // select first voodoo board in system - this means you
  // can use different boards in your computer connected to
  // different monitors from the same program! Glide
  // only works with the current board selected from
  // the grSstSelect function.
  grSstSelect( 0 );


  // sets 3dfx videomode with some parameters
  if (grSstWinOpen( 0,
               GR_RESOLUTION_800x600,
               GR_REFRESH_85Hz,
               GR_COLORFORMAT_RGBA,
               GR_ORIGIN_UPPER_LEFT,
               2, 1) == FXFALSE)
  {
    printf("Error! Couldn't set 3dfx videomode, you need a voodoo2 board to run this demo!");
    exit(1);
  }


  /*
   Load mipmap file tex.3df from disk:
   Retrieve texture info, this is required before the
   texture is loaded because we must know how big the
   texture is so we can allocate the required memory for
   it.
  */

  // get the texture info:
  if (gu3dfGetInfo("tex.3df",&TexInfo) == FXTRUE)
  {
  	// allocate memory for the mipmap:
  	TexInfo.data = (void*)malloc(TexInfo.mem_required);
  	// Load the mipmap from disk:
  	gu3dfLoad("tex.3df",&TexInfo);
  }
  else
  {
  	grGlideShutdown();
	printf("Couldn't load tex.3df file!");
	exit(1);
  }

  /*
    Once texture is loaded into memory we have to download
    the textures to texture memory, usually we should perform
    some memory management but since this only is a
    simple demo program we just downloads the mipmap levels on
    the texture units. For trilinear filtering we need to
	split the mipmap in the two texture units. One texture
	unit holds the even mipmap levels (2,4,6,...) and one texture unit
	holds the odd mipmap levels (1,3,5,...). The texture units is then
	configured for trinlinear filtering.
  */

  /*
    Take the information from the loaded mipmap and use it
    to set up a GrTexInfo structure which is used when we
    shall download the mipmap to texture memory on the
    texture units.
  */

  MipMap1.smallLod = TexInfo.header.small_lod;
  MipMap1.largeLod = TexInfo.header.large_lod;
  MipMap1.aspectRatio = TexInfo.header.aspect_ratio;
  MipMap1.data = TexInfo.data;
  MipMap1.format = TexInfo.header.format;

  // download the even mipmap levels to texture unit 1 and the
  // odd mipmap levels to texture unit 2:

  // we must find out the startaddresses of the texture memory on the
  // two texture units to be able to download the mipmaplevels on the
  // texture memory
  TexStartAddress1 = grTexMinAddress(GR_TMU0);
  TexStartAddress2 = grTexMinAddress(GR_TMU1);

  // the actual downloading goes here, start with downloading even mipmap
  // levels on texture unit 1...
  grTexDownloadMipMap(GR_TMU0,TexStartAddress1,
  	GR_MIPMAPLEVELMASK_EVEN,&MipMap1);

  // ...and downloads the odd mipmaplevels on texture unit 2.
  grTexDownloadMipMap(GR_TMU1,TexStartAddress2,
  	GR_MIPMAPLEVELMASK_ODD,&MipMap1);

  // After the mipmap have been downloaded to texture memory
  // we must tell the texture units where to look for the
  // mipmap and how it is composed, we do this here:

  // tell texture unit 1 we have put the even mipmaps at startaddress1...
  grTexSource(GR_TMU0,TexStartAddress1,GR_MIPMAPLEVELMASK_EVEN,
  	&MipMap1);

  // ...and tell texture unit 2 we have put the odd mipmaps at
  // startaddress2.
  grTexSource(GR_TMU1,TexStartAddress2,GR_MIPMAPLEVELMASK_ODD,
  	&MipMap1);

  // Now the hard part, we uses texture unit 2 as input to the
  // first texture unit and the first texture is configured
  // to output the trilinear filtered result to the color
  // combine unit.

  grTexCombine(GR_TMU1,GR_COMBINE_FUNCTION_LOCAL,GR_COMBINE_FACTOR_NONE,
  	GR_COMBINE_FUNCTION_LOCAL,GR_COMBINE_FACTOR_NONE,
	FXFALSE,FXFALSE);

  grTexCombine(GR_TMU0,
  	GR_COMBINE_FUNCTION_SCALE_MINUS_LOCAL_ADD_LOCAL,
 	GR_COMBINE_FACTOR_LOD_FRACTION,
    GR_COMBINE_FUNCTION_SCALE_MINUS_LOCAL_ADD_LOCAL,
	GR_COMBINE_FACTOR_LOD_FRACTION,
	FXFALSE,FXFALSE);

  // This function call have been modified in this demo - now the color
  // combine unit on the voodoo hardware is configured for gouraud shading
  // also (previous demo only used textured mapping without shading.
  grColorCombine(GR_COMBINE_FUNCTION_SCALE_OTHER,
  	GR_COMBINE_FACTOR_LOCAL,GR_COMBINE_LOCAL_ITERATED,
	GR_COMBINE_OTHER_TEXTURE,FXFALSE);


  // Now setup both texture units to use bilinear filtering for
  // magnification and for minification:
  grTexFilterMode(GR_TMU0,GR_TEXTUREFILTER_BILINEAR,
  	GR_TEXTUREFILTER_BILINEAR);
  grTexFilterMode(GR_TMU1,GR_TEXTUREFILTER_BILINEAR,
  	GR_TEXTUREFILTER_BILINEAR);

  // enter the main loop where the texture is zoomed in and out until the
  // user presses a key:
  w = 100.0;
  size = 0.0;
  increment = 2;
  while (1)
  {

    size += increment;

	if (size >= 200.0 && increment == 2)
   	  increment = -2;
	if (size <= 0.0 && increment == -2)
	  increment = 2;

	// Setup the vertices for the textured rectangle:
	// first the screen coordinates
	Rectangle[0].x = 100.0+size;
  	Rectangle[0].y = 100.0+size;
  	Rectangle[0].oow = 1/w;
	// then the texture coordinates - the texture coordinates spans
	// from 0.0/w to 256.0/w.
  	Rectangle[0].tmuvtx[0].sow = 0.0/w;
  	Rectangle[0].tmuvtx[1].sow = 0.0/w;
  	Rectangle[0].tmuvtx[0].tow = 0.0/w;
  	Rectangle[0].tmuvtx[1].tow = 0.0/w;
  	Rectangle[0].tmuvtx[0].oow = 1/w;
  	Rectangle[0].tmuvtx[1].oow = 1/w;
	// In this demo we uses the r,g,b components to, required for
    // the gouraud shading.
	Rectangle[0].r = 255-size;
	Rectangle[0].g = 0;
	Rectangle[0].b = 0;

  	Rectangle[1].x = 700.0-size;
  	Rectangle[1].y = 100.0+size;
  	Rectangle[1].oow = 1/w;
  	Rectangle[1].tmuvtx[0].sow = 256.0/w;
  	Rectangle[1].tmuvtx[1].sow = 256.0/w;
  	Rectangle[1].tmuvtx[0].tow = 0.0/w;
  	Rectangle[1].tmuvtx[1].tow = 0.0/w;
  	Rectangle[1].tmuvtx[0].oow = 1/w;
  	Rectangle[1].tmuvtx[1].oow = 1/w;
	Rectangle[1].r = 0;
	Rectangle[1].g = 255-size;
	Rectangle[1].b = 0;

  	Rectangle[2].x = 700.0-size;
  	Rectangle[2].y = 500.0-size;
  	Rectangle[2].oow = 1/w;
  	Rectangle[2].tmuvtx[0].sow = 256.0/w;
  	Rectangle[2].tmuvtx[1].sow = 256.0/w;
  	Rectangle[2].tmuvtx[0].tow = 128.0/w;
  	Rectangle[2].tmuvtx[1].tow = 128.0/w;
  	Rectangle[2].tmuvtx[0].oow = 1/w;
  	Rectangle[2].tmuvtx[1].oow = 1/w;
	Rectangle[2].r = 0;
	Rectangle[2].g = 0;
	Rectangle[2].b = 255-size;

  	Rectangle[3].x = 100.0+size;
  	Rectangle[3].y = 500.0-size;
  	Rectangle[3].oow = 1/w;
  	Rectangle[3].tmuvtx[0].sow = 0.0/w;
  	Rectangle[3].tmuvtx[1].sow = 0.0/w;
  	Rectangle[3].tmuvtx[0].tow = 128.0/w;
  	Rectangle[3].tmuvtx[1].tow = 128.0/w;
    Rectangle[3].tmuvtx[0].oow = 1/w;
    Rectangle[3].tmuvtx[1].oow = 1/w;
	Rectangle[3].r = 255-size;
	Rectangle[3].g = 255-size;
	Rectangle[3].b = 255-size;

    vlist[0] = 0;
    vlist[1] = 1;
    vlist[2] = 2;
    vlist[3] = 3;


  	// clears the backbuffer black
  	grBufferClear( 0x00000000, 0, GR_WDEPTHVALUE_FARTHEST );

	// FINALLY draw the polygon:
    grDrawPolygon(4,vlist,Rectangle);

    // swaps the backbuffer to the frontbuffer - makes the
    // trilinear filtered, gouraud shaded rectangle visible.
    grBufferSwap(1);

  	if (kbhit())
  	{
    	// get key from queue:
		getch();
  		// shut down glide and return to normal videomode:
  		grGlideShutdown();
		exit(1);
	}
  }
}

